/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

import QtQuick 2.11
import QtQuick.Window 2.11
import QtQuick.Controls 1.4

import snakesimulator 1.0

ApplicationWindow {
    id: root
    visible: true
    property int tileSize: 20
    width: field.width*tileSize
    height: field.height*tileSize

    Rectangle {
        color: "#000000"
        anchors.fill: parent
    }

    Repeater {
        model: snake.pointsData.length
        Rectangle {
            color: "#ffffff"
            x: snake.pointsData[model.index].x*tileSize
            y: snake.pointsData[model.index].y*tileSize
            width: tileSize
            height: tileSize
        }
    }

    Rectangle {
        color: "#eeffee"
        x: field.food.x*tileSize
        y: field.food.y*tileSize
        width: tileSize
        height: tileSize
    }

    Column {
        anchors {
            right: parent.right
            top: parent.top
            margins: 10
        }
        Text {
            color: "#ddffee"
            text: "Generation: " + stats.generation
        }
        Text {
            color: "#ddffee"
            text: "Individual: " + stats.individual
        }
        Text {
            color: "#ddffee"
            text: "Move: " + stats.move
        }
        TextField {
            onAccepted:  {
                client.setSpeed(parseInt(text, 10))
            }
        }
    }

    Connections {
        target: field
        onWidthChanged: {
            console.log("New width: " + field.width)
        }
    }
}
